@@ -12,7 +12,7 @@ G_BEGIN_DECLS
 
 #define E_TYPE_BOOK_QUERY (e_book_query_get_type ())
 
-typedef struct EBookQuery EBookQuery;
+typedef struct _EBookQuery EBookQuery;
 
 /**
  * EBookQueryTest:
@@ -20,20 +20,19 @@ typedef struct EBookQuery EBookQuery;
  * @E_BOOK_QUERY_CONTAINS: check if a field contains the test value
  * @E_BOOK_QUERY_BEGINS_WITH: check if a field starts with the test value
  * @E_BOOK_QUERY_ENDS_WITH: check if a field ends with the test value
- * @E_BOOK_QUERY_EQUALS_PHONE_NUMBER: check that a field and the test value
- * match exactly when interpreted as phone number, that is after stripping
- * formatting like dashes, dots and spaces. See E_PHONE_NUMBER_MATCH_EXACT.
- * @E_BOOK_QUERY_EQUALS_NATIONAL_PHONE_NUMBER: check that a field and the
- * test value match when interpreted as phone number, except for the
- * (omitted) country calling code.
- * @E_BOOK_QUERY_EQUALS_SHORT_PHONE_NUMBER: check that a field and the test
- * value match is the sense that both values appear to be phone numbers,
- * and one might be a part (suffix) of the other.
+ * @E_BOOK_QUERY_EQUALS_PHONE_NUMBER: check if a field matches with a value tested
+ * using e_phone_number_compare_strings(), the match must be of strenth %E_PHONE_NUMBER_MATCH_EXACT
+ * for this query to return any matches.
+ * @E_BOOK_QUERY_EQUALS_NATIONAL_PHONE_NUMBER: check if a field matches with a value tested
+ * using e_phone_number_compare_strings(), the match must be at least of strength %E_PHONE_NUMBER_MATCH_NATIONAL
+ * for this query to return any matches.
+ * @E_BOOK_QUERY_EQUALS_SHORT_PHONE_NUMBER: check if a field matches with a value tested
+ * using e_phone_number_compare_strings(), the match must be at least of strength %E_PHONE_NUMBER_MATCH_SHORT
+ * for this query to return any matches.
  *
  * The kind of test a query created by e_book_query_field_test() shall perform.
  *
- * See also: E_PHONE_NUMBER_MATCH_EXACT, E_PHONE_NUMBER_MATCH_NATIONAL and
- * E_PHONE_NUMBER_MATCH_SHORT.
+ * See also: e_phone_number_compare_strings().
  **/
 typedef enum {
   E_BOOK_QUERY_IS,
