@@ -48,6 +48,9 @@ struct _EBookClientViewPrivate {
 	guint running : 1;
 	guint complete : 1;
 
+	GMainContext *main_context;
+	GMutex main_context_lock;
+
 	EDataBook *direct_book;
 
 	gulong objects_added_handler_id;
@@ -123,6 +126,56 @@ signal_closure_free (SignalClosure *signal_closure)
 	g_slice_free (SignalClosure, signal_closure);
 }
 
+static GWeakRef *
+weak_ref_new (gpointer object)
+{
+	GWeakRef *weak_ref;
+
+	weak_ref = g_slice_new0 (GWeakRef);
+	g_weak_ref_set (weak_ref, object);
+
+	return weak_ref;
+}
+
+static void
+weak_ref_free (GWeakRef *weak_ref)
+{
+	g_weak_ref_set (weak_ref, NULL);
+	g_slice_free (GWeakRef, weak_ref);
+}
+
+static GMainContext *
+book_client_view_ref_main_context (EBookClientView *client_view)
+{
+	GMainContext *main_context;
+
+	/* Intentionally not checking for NULL so we get a console
+	 * warning if we try to reference a NULL main context, but
+	 * that should never happen. */
+
+	g_mutex_lock (&client_view->priv->main_context_lock);
+
+	main_context = g_main_context_ref (client_view->priv->main_context);
+
+	g_mutex_unlock (&client_view->priv->main_context_lock);
+
+	return main_context;
+}
+
+static void
+book_client_view_set_main_context (EBookClientView *client_view,
+                                   GMainContext *main_context)
+{
+	g_mutex_lock (&client_view->priv->main_context_lock);
+
+	if (client_view->priv->main_context != NULL)
+		g_main_context_unref (client_view->priv->main_context);
+
+	client_view->priv->main_context = g_main_context_ref (main_context);
+
+	g_mutex_unlock (&client_view->priv->main_context_lock);
+}
+
 static gboolean
 book_client_view_emit_objects_added_idle_cb (gpointer user_data)
 {
@@ -223,7 +276,6 @@ static void
 book_client_view_emit_objects_added (EBookClientView *client_view,
                                      GSList *object_list)
 {
-	EBookClient *client;
 	GSource *idle_source;
 	GMainContext *main_context;
 	SignalClosure *signal_closure;
@@ -232,8 +284,7 @@ book_client_view_emit_objects_added (EBookClientView *client_view,
 	g_weak_ref_set (&signal_closure->client_view, client_view);
 	signal_closure->object_list = object_list;  /* takes ownership */
 
-	client = e_book_client_view_get_client (client_view);
-	main_context = e_client_ref_main_context (E_CLIENT (client));
+	main_context = book_client_view_ref_main_context (client_view);
 
 	idle_source = g_idle_source_new ();
 	g_source_set_callback (
@@ -251,7 +302,6 @@ static void
 book_client_view_emit_objects_modified (EBookClientView *client_view,
                                         GSList *object_list)
 {
-	EBookClient *client;
 	GSource *idle_source;
 	GMainContext *main_context;
 	SignalClosure *signal_closure;
@@ -260,8 +310,7 @@ book_client_view_emit_objects_modified (EBookClientView *client_view,
 	g_weak_ref_set (&signal_closure->client_view, client_view);
 	signal_closure->object_list = object_list;  /* takes ownership */
 
-	client = e_book_client_view_get_client (client_view);
-	main_context = e_client_ref_main_context (E_CLIENT (client));
+	main_context = book_client_view_ref_main_context (client_view);
 
 	idle_source = g_idle_source_new ();
 	g_source_set_callback (
@@ -305,18 +354,18 @@ direct_contacts_ready (GObject *source_object,
 {
 	NotificationData *data = (NotificationData *) user_data;
 	GSList *contacts = NULL;
-	GError *error = NULL;
+	GError *local_error = NULL;
 
 	e_data_book_get_contacts_finish (
 		E_DATA_BOOK (source_object),
-		result, &contacts, &error);
+		result, &contacts, &local_error);
 
-	if (error != NULL) {
+	if (local_error != NULL) {
 		g_warn_if_fail (contacts == NULL);
 		g_warning (
 			"Error fetching contacts directly: %s\n",
-			error->message);
-		g_error_free (error);
+			local_error->message);
+		g_error_free (local_error);
 
 	} else if (data->signum == OBJECTS_ADDED) {
 		/* Takes ownership of the linked list. */
@@ -347,18 +396,18 @@ direct_contacts_fetch (EBookClientView *view,
 	 */
 	if (!view->priv->complete) {
 		GSList *contacts = NULL;
-		GError *error = NULL;
+		GError *local_error = NULL;
 
 		e_data_book_get_contacts_sync (
-			view->priv->direct_book,
-			sexp, &contacts, NULL, &error);
+			view->priv->direct_book, sexp,
+			&contacts, NULL, &local_error);
 
-		if (error != NULL) {
+		if (local_error != NULL) {
 			g_warn_if_fail (contacts == NULL);
 			g_warning (
 				"Error fetching contacts directly: %s\n",
-				error->message);
-			g_error_free (error);
+				local_error->message);
+			g_error_free (local_error);
 
 		} else if (signum == OBJECTS_ADDED) {
 			/* Takes ownership of the linked list. */
@@ -392,172 +441,221 @@ direct_contacts_fetch (EBookClientView *view,
 static void
 book_client_view_objects_added_cb (EGdbusBookView *object,
                                    const gchar * const *vcards,
-                                   EBookClientView *client_view)
+                                   GWeakRef *client_view_weak_ref)
 {
-	GSList *list = NULL;
-	gint ii;
+	EBookClientView *client_view;
 
-	if (!client_view->priv->running)
-		return;
+	client_view = g_weak_ref_get (client_view_weak_ref);
 
-	/* array contains UIDs only */
-	if (client_view->priv->direct_book != NULL) {
-		direct_contacts_fetch (client_view, vcards, OBJECTS_ADDED);
-		return;
+	if (client_view != NULL) {
+		GSList *list = NULL;
+		gint ii;
+
+		if (!client_view->priv->running) {
+			g_object_unref (client_view);
+			return;
+		}
+
+		/* array contains UIDs only */
+		if (client_view->priv->direct_book != NULL) {
+			direct_contacts_fetch (
+				client_view, vcards, OBJECTS_ADDED);
+			g_object_unref (client_view);
+			return;
+		}
+
+		/* array contains both UID and vcard */
+		for (ii = 0; vcards[ii] != NULL && vcards[ii + 1] != NULL; ii += 2) {
+			EContact *contact;
+			const gchar *vcard = vcards[ii];
+			const gchar *uid = vcards[ii + 1];
+
+			contact = e_contact_new_from_vcard_with_uid (vcard, uid);
+			list = g_slist_prepend (list, contact);
+		}
+
+		list = g_slist_reverse (list);
+
+		/* Takes ownership of the linked list. */
+		book_client_view_emit_objects_added (client_view, list);
+
+		g_object_unref (client_view);
 	}
-
-	/* array contains both UID and vcard */
-	for (ii = 0; vcards[ii] != NULL && vcards[ii + 1] != NULL; ii += 2) {
-		EContact *contact;
-		const gchar *vcard = vcards[ii];
-		const gchar *uid = vcards[ii + 1];
-
-		contact = e_contact_new_from_vcard_with_uid (vcard, uid);
-		list = g_slist_prepend (list, contact);
-	}
-
-	list = g_slist_reverse (list);
-
-	/* Takes ownership of the linked list. */
-	book_client_view_emit_objects_added (client_view, list);
 }
 
 static void
 book_client_view_objects_modified_cb (EGdbusBookView *object,
                                       const gchar * const *vcards,
-                                      EBookClientView *client_view)
+                                      GWeakRef *client_view_weak_ref)
 {
-	GSList *list = NULL;
-	gint ii;
+	EBookClientView *client_view;
 
-	if (!client_view->priv->running)
-		return;
+	client_view = g_weak_ref_get (client_view_weak_ref);
 
-	/* array contains UIDs only */
-	if (client_view->priv->direct_book != NULL) {
-		direct_contacts_fetch (client_view, vcards, OBJECTS_MODIFIED);
-		return;
+	if (client_view != NULL) {
+		GSList *list = NULL;
+		gint ii;
+
+		if (!client_view->priv->running) {
+			g_object_unref (client_view);
+			return;
+		}
+
+		/* array contains UIDs only */
+		if (client_view->priv->direct_book != NULL) {
+			direct_contacts_fetch (
+				client_view, vcards, OBJECTS_MODIFIED);
+			g_object_unref (client_view);
+			return;
+		}
+
+		/* array contains both UID and vcard */
+		for (ii = 0; vcards[ii] != NULL && vcards[ii + 1] != NULL; ii += 2) {
+			EContact *contact;
+			const gchar *vcard = vcards[ii];
+			const gchar *uid = vcards[ii + 1];
+
+			contact = e_contact_new_from_vcard_with_uid (vcard, uid);
+			list = g_slist_prepend (list, contact);
+		}
+
+		list = g_slist_reverse (list);
+
+		/* Takes ownership of the linked list. */
+		book_client_view_emit_objects_modified (client_view, list);
+
+		g_object_unref (client_view);
 	}
-
-	/* array contains both UID and vcard */
-	for (ii = 0; vcards[ii] != NULL && vcards[ii + 1] != NULL; ii += 2) {
-		EContact *contact;
-		const gchar *vcard = vcards[ii];
-		const gchar *uid = vcards[ii + 1];
-
-		contact = e_contact_new_from_vcard_with_uid (vcard, uid);
-		list = g_slist_prepend (list, contact);
-	}
-
-	list = g_slist_reverse (list);
-
-	/* Takes ownership of the linked list. */
-	book_client_view_emit_objects_modified (client_view, list);
 }
 
 static void
 book_client_view_objects_removed_cb (EGdbusBookView *object,
                                      const gchar * const *ids,
-                                     EBookClientView *client_view)
+                                     GWeakRef *client_view_weak_ref)
 {
-	EBookClient *client;
-	GSource *idle_source;
-	GMainContext *main_context;
-	SignalClosure *signal_closure;
-	GSList *list = NULL;
-	gint ii;
+	EBookClientView *client_view;
 
-	if (!client_view->priv->running)
-		return;
+	client_view = g_weak_ref_get (client_view_weak_ref);
 
-	for (ii = 0; ids[ii] != NULL; ii++)
-		list = g_slist_prepend (list, g_strdup (ids[ii]));
+	if (client_view != NULL) {
+		GSource *idle_source;
+		GMainContext *main_context;
+		SignalClosure *signal_closure;
+		GSList *list = NULL;
+		gint ii;
 
-	signal_closure = g_slice_new0 (SignalClosure);
-	g_weak_ref_set (&signal_closure->client_view, client_view);
-	signal_closure->string_list = g_slist_reverse (list);
+		if (!client_view->priv->running) {
+			g_object_unref (client_view);
+			return;
+		}
 
-	client = e_book_client_view_get_client (client_view);
-	main_context = e_client_ref_main_context (E_CLIENT (client));
+		for (ii = 0; ids[ii] != NULL; ii++)
+			list = g_slist_prepend (list, g_strdup (ids[ii]));
 
-	idle_source = g_idle_source_new ();
-	g_source_set_callback (
-		idle_source,
-		book_client_view_emit_objects_removed_idle_cb,
-		signal_closure,
-		(GDestroyNotify) signal_closure_free);
-	g_source_attach (idle_source, main_context);
-	g_source_unref (idle_source);
+		signal_closure = g_slice_new0 (SignalClosure);
+		g_weak_ref_set (&signal_closure->client_view, client_view);
+		signal_closure->string_list = g_slist_reverse (list);
 
-	g_main_context_unref (main_context);
+		main_context = book_client_view_ref_main_context (client_view);
+
+		idle_source = g_idle_source_new ();
+		g_source_set_callback (
+			idle_source,
+			book_client_view_emit_objects_removed_idle_cb,
+			signal_closure,
+			(GDestroyNotify) signal_closure_free);
+		g_source_attach (idle_source, main_context);
+		g_source_unref (idle_source);
+
+		g_main_context_unref (main_context);
+
+		g_object_unref (client_view);
+	}
 }
 
 static void
 book_client_view_progress_cb (EGdbusBookView *object,
                               guint percent,
                               const gchar *message,
-                              EBookClientView *client_view)
+                              GWeakRef *client_view_weak_ref)
 {
-	EBookClient *client;
-	GSource *idle_source;
-	GMainContext *main_context;
-	SignalClosure *signal_closure;
+	EBookClientView *client_view;
 
-	if (!client_view->priv->running)
-		return;
+	client_view = g_weak_ref_get (client_view_weak_ref);
 
-	signal_closure = g_slice_new0 (SignalClosure);
-	g_weak_ref_set (&signal_closure->client_view, client_view);
-	signal_closure->message = g_strdup (message);
-	signal_closure->percent = percent;
+	if (client_view != NULL) {
+		GSource *idle_source;
+		GMainContext *main_context;
+		SignalClosure *signal_closure;
 
-	client = e_book_client_view_get_client (client_view);
-	main_context = e_client_ref_main_context (E_CLIENT (client));
+		if (!client_view->priv->running) {
+			g_object_unref (client_view);
+			return;
+		}
 
-	idle_source = g_idle_source_new ();
-	g_source_set_callback (
-		idle_source,
-		book_client_view_emit_progress_idle_cb,
-		signal_closure,
-		(GDestroyNotify) signal_closure_free);
-	g_source_attach (idle_source, main_context);
-	g_source_unref (idle_source);
+		signal_closure = g_slice_new0 (SignalClosure);
+		g_weak_ref_set (&signal_closure->client_view, client_view);
+		signal_closure->message = g_strdup (message);
+		signal_closure->percent = percent;
 
-	g_main_context_unref (main_context);
+		main_context = book_client_view_ref_main_context (client_view);
+
+		idle_source = g_idle_source_new ();
+		g_source_set_callback (
+			idle_source,
+			book_client_view_emit_progress_idle_cb,
+			signal_closure,
+			(GDestroyNotify) signal_closure_free);
+		g_source_attach (idle_source, main_context);
+		g_source_unref (idle_source);
+
+		g_main_context_unref (main_context);
+
+		g_object_unref (client_view);
+	}
 }
 
 static void
 book_client_view_complete_cb (EGdbusBookView *object,
                               const gchar * const *in_error_strv,
-                              EBookClientView *client_view)
+                              GWeakRef *client_view_weak_ref)
 {
-	EBookClient *client;
-	GSource *idle_source;
-	GMainContext *main_context;
-	SignalClosure *signal_closure;
+	EBookClientView *client_view;
 
-	if (!client_view->priv->running)
-		return;
+	client_view = g_weak_ref_get (client_view_weak_ref);
 
-	signal_closure = g_slice_new0 (SignalClosure);
-	g_weak_ref_set (&signal_closure->client_view, client_view);
-	e_gdbus_templates_decode_error (in_error_strv, &signal_closure->error);
+	if (client_view != NULL) {
+		GSource *idle_source;
+		GMainContext *main_context;
+		SignalClosure *signal_closure;
 
-	client = e_book_client_view_get_client (client_view);
-	main_context = e_client_ref_main_context (E_CLIENT (client));
+		if (!client_view->priv->running) {
+			g_object_unref (client_view);
+			return;
+		}
 
-	idle_source = g_idle_source_new ();
-	g_source_set_callback (
-		idle_source,
-		book_client_view_emit_complete_idle_cb,
-		signal_closure,
-		(GDestroyNotify) signal_closure_free);
-	g_source_attach (idle_source, main_context);
-	g_source_unref (idle_source);
+		signal_closure = g_slice_new0 (SignalClosure);
+		g_weak_ref_set (&signal_closure->client_view, client_view);
+		e_gdbus_templates_decode_error (
+			in_error_strv, &signal_closure->error);
 
-	g_main_context_unref (main_context);
+		main_context = book_client_view_ref_main_context (client_view);
 
-	client_view->priv->complete = TRUE;
+		idle_source = g_idle_source_new ();
+		g_source_set_callback (
+			idle_source,
+			book_client_view_emit_complete_idle_cb,
+			signal_closure,
+			(GDestroyNotify) signal_closure_free);
+		g_source_attach (idle_source, main_context);
+		g_source_unref (idle_source);
+
+		g_main_context_unref (main_context);
+
+		client_view->priv->complete = TRUE;
+
+		g_object_unref (client_view);
+	}
 }
 
 static void
@@ -565,14 +663,15 @@ book_client_view_dispose_cb (GObject *source_object,
                              GAsyncResult *result,
                              gpointer user_data)
 {
-	GError *error = NULL;
+	GError *local_error = NULL;
 
 	e_gdbus_book_view_call_dispose_finish (
-		G_DBUS_PROXY (source_object), result, &error);
+		G_DBUS_PROXY (source_object), result, &local_error);
 
-	if (error != NULL) {
-		g_warning ("%s: %s", G_STRFUNC, error->message);
-		g_error_free (error);
+	if (local_error != NULL) {
+		g_dbus_error_strip_remote_error (local_error);
+		g_warning ("%s: %s", G_STRFUNC, local_error->message);
+		g_error_free (local_error);
 	}
 }
 
@@ -701,6 +800,11 @@ book_client_view_dispose (GObject *object)
 		priv->connection = NULL;
 	}
 
+	if (priv->main_context != NULL) {
+		g_main_context_unref (priv->main_context);
+		priv->main_context = NULL;
+	}
+
 	if (priv->direct_book != NULL) {
 		g_object_unref (priv->direct_book);
 		priv->direct_book = NULL;
@@ -745,6 +849,8 @@ book_client_view_finalize (GObject *object)
 
 	g_free (priv->object_path);
 
+	g_mutex_clear (&priv->main_context_lock);
+
 	/* Chain up to parent's finalize() method. */
 	G_OBJECT_CLASS (e_book_client_view_parent_class)->finalize (object);
 }
@@ -772,29 +878,39 @@ book_client_view_initable_init (GInitable *initable,
 
 	priv->dbus_proxy = G_DBUS_PROXY (gdbus_bookview);
 
-	handler_id = g_signal_connect (
+	handler_id = g_signal_connect_data (
 		priv->dbus_proxy, "objects-added",
-		G_CALLBACK (book_client_view_objects_added_cb), initable);
+		G_CALLBACK (book_client_view_objects_added_cb),
+		weak_ref_new (initable),
+		(GClosureNotify) weak_ref_free, 0);
 	priv->objects_added_handler_id = handler_id;
 
-	handler_id = g_signal_connect (
+	handler_id = g_signal_connect_data (
 		priv->dbus_proxy, "objects-modified",
-		G_CALLBACK (book_client_view_objects_modified_cb), initable);
+		G_CALLBACK (book_client_view_objects_modified_cb),
+		weak_ref_new (initable),
+		(GClosureNotify) weak_ref_free, 0);
 	priv->objects_modified_handler_id = handler_id;
 
-	handler_id = g_signal_connect (
+	handler_id = g_signal_connect_data (
 		priv->dbus_proxy, "objects-removed",
-		G_CALLBACK (book_client_view_objects_removed_cb), initable);
+		G_CALLBACK (book_client_view_objects_removed_cb),
+		weak_ref_new (initable),
+		(GClosureNotify) weak_ref_free, 0);
 	priv->objects_removed_handler_id = handler_id;
 
-	handler_id = g_signal_connect (
+	handler_id = g_signal_connect_data (
 		priv->dbus_proxy, "progress",
-		G_CALLBACK (book_client_view_progress_cb), initable);
+		G_CALLBACK (book_client_view_progress_cb),
+		weak_ref_new (initable),
+		(GClosureNotify) weak_ref_free, 0);
 	priv->progress_handler_id = handler_id;
 
-	handler_id = g_signal_connect (
+	handler_id = g_signal_connect_data (
 		priv->dbus_proxy, "complete",
-		G_CALLBACK (book_client_view_complete_cb), initable);
+		G_CALLBACK (book_client_view_complete_cb),
+		weak_ref_new (initable),
+		(GClosureNotify) weak_ref_free, 0);
 	priv->complete_handler_id = handler_id;
 
 	/* When in direct read access mode, we add a special field
@@ -934,6 +1050,8 @@ static void
 e_book_client_view_init (EBookClientView *view)
 {
 	view->priv = E_BOOK_CLIENT_VIEW_GET_PRIVATE (view);
+
+	g_mutex_init (&view->priv->main_context_lock);
 }
 
 /**
@@ -999,20 +1117,26 @@ void
 e_book_client_view_start (EBookClientView *view,
                           GError **error)
 {
-	gboolean success;
+	GMainContext *main_context;
 	GError *local_error = NULL;
 
 	g_return_if_fail (E_IS_BOOK_CLIENT_VIEW (view));
 
+	/* Emit signals from the current thread-default main context. */
+	main_context = g_main_context_ref_thread_default ();
+	book_client_view_set_main_context (view, main_context);
+	g_main_context_unref (main_context);
+
 	view->priv->running = TRUE;
 
-	success = e_gdbus_book_view_call_start_sync (
+	e_gdbus_book_view_call_start_sync (
 		view->priv->dbus_proxy, NULL, &local_error);
-	if (!success)
-		view->priv->running = FALSE;
 
-	e_client_unwrap_dbus_error (
-		E_CLIENT (view->priv->client), local_error, error);
+	if (local_error != NULL) {
+		view->priv->running = FALSE;
+		g_dbus_error_strip_remote_error (local_error);
+		g_propagate_error (error, local_error);
+	}
 }
 
 /**
@@ -1035,8 +1159,10 @@ e_book_client_view_stop (EBookClientView *view,
 	e_gdbus_book_view_call_stop_sync (
 		view->priv->dbus_proxy, NULL, &local_error);
 
-	e_client_unwrap_dbus_error (
-		E_CLIENT (view->priv->client), local_error, error);
+	if (local_error != NULL) {
+		g_dbus_error_strip_remote_error (local_error);
+		g_propagate_error (error, local_error);
+	}
 }
 
 /**
@@ -1061,8 +1187,10 @@ e_book_client_view_set_flags (EBookClientView *view,
 	e_gdbus_book_view_call_set_flags_sync (
 		view->priv->dbus_proxy, flags, NULL, &local_error);
 
-	e_client_unwrap_dbus_error (
-		E_CLIENT (view->priv->client), local_error, error);
+	if (local_error != NULL) {
+		g_dbus_error_strip_remote_error (local_error);
+		g_propagate_error (error, local_error);
+	}
 }
 
 /**
@@ -1113,7 +1241,9 @@ e_book_client_view_set_fields_of_interest (EBookClientView *view,
 		NULL, &local_error);
 	g_strfreev (strv);
 
-	e_client_unwrap_dbus_error (
-		E_CLIENT (view->priv->client), local_error, error);
+	if (local_error != NULL) {
+		g_dbus_error_strip_remote_error (local_error);
+		g_propagate_error (error, local_error);
+	}
 }
 
