@@ -48,6 +48,9 @@ struct _ECalClientViewPrivate {
 	gchar *object_path;
 	gboolean running;
 
+	GMainContext *main_context;
+	GMutex main_context_lock;
+
 	gulong objects_added_handler_id;
 	gulong objects_modified_handler_id;
 	gulong objects_removed_handler_id;
@@ -114,6 +117,56 @@ signal_closure_free (SignalClosure *signal_closure)
 	g_slice_free (SignalClosure, signal_closure);
 }
 
+static GWeakRef *
+weak_ref_new (gpointer object)
+{
+	GWeakRef *weak_ref;
+
+	weak_ref = g_slice_new0 (GWeakRef);
+	g_weak_ref_set (weak_ref, object);
+
+	return weak_ref;
+}
+
+static void
+weak_ref_free (GWeakRef *weak_ref)
+{
+	g_weak_ref_set (weak_ref, NULL);
+	g_slice_free (GWeakRef, weak_ref);
+}
+
+static GMainContext *
+cal_client_view_ref_main_context (ECalClientView *client_view)
+{
+	GMainContext *main_context;
+
+	/* Intentionally not checking for NULL so we get a console
+	 * warning if we try to reference a NULL main context, but
+	 * that should never happen. */
+
+	g_mutex_lock (&client_view->priv->main_context_lock);
+
+	main_context = g_main_context_ref (client_view->priv->main_context);
+
+	g_mutex_unlock (&client_view->priv->main_context_lock);
+
+	return main_context;
+}
+
+static void
+cal_client_view_set_main_context (ECalClientView *client_view,
+                                  GMainContext *main_context)
+{
+	g_mutex_lock (&client_view->priv->main_context_lock);
+
+	if (client_view->priv->main_context != NULL)
+		g_main_context_unref (client_view->priv->main_context);
+
+	client_view->priv->main_context = g_main_context_ref (main_context);
+
+	g_mutex_unlock (&client_view->priv->main_context_lock);
+}
+
 static GSList *
 build_object_list (const gchar * const *seq)
 {
@@ -262,163 +315,204 @@ cal_client_view_emit_complete_idle_cb (gpointer user_data)
 static void
 cal_client_view_objects_added_cb (EGdbusCalView *dbus_proxy,
                                   const gchar * const *objects,
-                                  ECalClientView *client_view)
+                                  GWeakRef *client_view_weak_ref)
 {
-	ECalClient *client;
-	GSource *idle_source;
-	GMainContext *main_context;
-	SignalClosure *signal_closure;
+	ECalClientView *client_view;
 
-	if (!client_view->priv->running)
-		return;
+	client_view = g_weak_ref_get (client_view_weak_ref);
 
-	signal_closure = g_slice_new0 (SignalClosure);
-	g_weak_ref_set (&signal_closure->client_view, client_view);
-	signal_closure->component_list = build_object_list (objects);
+	if (client_view != NULL) {
+		GSource *idle_source;
+		GMainContext *main_context;
+		SignalClosure *signal_closure;
 
-	client = e_cal_client_view_get_client (client_view);
-	main_context = e_client_ref_main_context (E_CLIENT (client));
+		if (!client_view->priv->running) {
+			g_object_unref (client_view);
+			return;
+		}
 
-	idle_source = g_idle_source_new ();
-	g_source_set_callback (
-		idle_source,
-		cal_client_view_emit_objects_added_idle_cb,
-		signal_closure,
-		(GDestroyNotify) signal_closure_free);
-	g_source_attach (idle_source, main_context);
-	g_source_unref (idle_source);
+		signal_closure = g_slice_new0 (SignalClosure);
+		g_weak_ref_set (&signal_closure->client_view, client_view);
+		signal_closure->component_list = build_object_list (objects);
 
-	g_main_context_unref (main_context);
+		main_context = cal_client_view_ref_main_context (client_view);
+
+		idle_source = g_idle_source_new ();
+		g_source_set_callback (
+			idle_source,
+			cal_client_view_emit_objects_added_idle_cb,
+			signal_closure,
+			(GDestroyNotify) signal_closure_free);
+		g_source_attach (idle_source, main_context);
+		g_source_unref (idle_source);
+
+		g_main_context_unref (main_context);
+
+		g_object_unref (client_view);
+	}
 }
 
 static void
 cal_client_view_objects_modified_cb (EGdbusCalView *dbus_proxy,
                                      const gchar * const *objects,
-                                     ECalClientView *client_view)
+                                     GWeakRef *client_view_weak_ref)
 {
-	ECalClient *client;
-	GSource *idle_source;
-	GMainContext *main_context;
-	SignalClosure *signal_closure;
+	ECalClientView *client_view;
 
-	if (!client_view->priv->running)
-		return;
+	client_view = g_weak_ref_get (client_view_weak_ref);
 
-	signal_closure = g_slice_new0 (SignalClosure);
-	g_weak_ref_set (&signal_closure->client_view, client_view);
-	signal_closure->component_list = build_object_list (objects);
+	if (client_view != NULL) {
+		GSource *idle_source;
+		GMainContext *main_context;
+		SignalClosure *signal_closure;
 
-	client = e_cal_client_view_get_client (client_view);
-	main_context = e_client_ref_main_context (E_CLIENT (client));
+		if (!client_view->priv->running) {
+			g_object_unref (client_view);
+			return;
+		}
 
-	idle_source = g_idle_source_new ();
-	g_source_set_callback (
-		idle_source,
-		cal_client_view_emit_objects_modified_idle_cb,
-		signal_closure,
-		(GDestroyNotify) signal_closure_free);
-	g_source_attach (idle_source, main_context);
-	g_source_unref (idle_source);
+		signal_closure = g_slice_new0 (SignalClosure);
+		g_weak_ref_set (&signal_closure->client_view, client_view);
+		signal_closure->component_list = build_object_list (objects);
 
-	g_main_context_unref (main_context);
+		main_context = cal_client_view_ref_main_context (client_view);
+
+		idle_source = g_idle_source_new ();
+		g_source_set_callback (
+			idle_source,
+			cal_client_view_emit_objects_modified_idle_cb,
+			signal_closure,
+			(GDestroyNotify) signal_closure_free);
+		g_source_attach (idle_source, main_context);
+		g_source_unref (idle_source);
+
+		g_main_context_unref (main_context);
+
+		g_object_unref (client_view);
+	}
 }
 
 static void
 cal_client_view_objects_removed_cb (EGdbusCalView *dbus_proxy,
                                     const gchar * const *uids,
-                                    ECalClientView *client_view)
+                                    GWeakRef *client_view_weak_ref)
 {
-	ECalClient *client;
-	GSource *idle_source;
-	GMainContext *main_context;
-	SignalClosure *signal_closure;
+	ECalClientView *client_view;
 
-	if (!client_view->priv->running)
-		return;
+	client_view = g_weak_ref_get (client_view_weak_ref);
 
-	signal_closure = g_slice_new0 (SignalClosure);
-	g_weak_ref_set (&signal_closure->client_view, client_view);
-	signal_closure->component_id_list = build_id_list (uids);
+	if (client_view != NULL) {
+		GSource *idle_source;
+		GMainContext *main_context;
+		SignalClosure *signal_closure;
 
-	client = e_cal_client_view_get_client (client_view);
-	main_context = e_client_ref_main_context (E_CLIENT (client));
+		if (!client_view->priv->running) {
+			g_object_unref (client_view);
+			return;
+		}
 
-	idle_source = g_idle_source_new ();
-	g_source_set_callback (
-		idle_source,
-		cal_client_view_emit_objects_removed_idle_cb,
-		signal_closure,
-		(GDestroyNotify) signal_closure_free);
-	g_source_attach (idle_source, main_context);
-	g_source_unref (idle_source);
+		signal_closure = g_slice_new0 (SignalClosure);
+		g_weak_ref_set (&signal_closure->client_view, client_view);
+		signal_closure->component_id_list = build_id_list (uids);
 
-	g_main_context_unref (main_context);
+		main_context = cal_client_view_ref_main_context (client_view);
+
+		idle_source = g_idle_source_new ();
+		g_source_set_callback (
+			idle_source,
+			cal_client_view_emit_objects_removed_idle_cb,
+			signal_closure,
+			(GDestroyNotify) signal_closure_free);
+		g_source_attach (idle_source, main_context);
+		g_source_unref (idle_source);
+
+		g_main_context_unref (main_context);
+
+		g_object_unref (client_view);
+	}
 }
 
 static void
 cal_client_view_progress_cb (EGdbusCalView *dbus_proxy,
                              guint percent,
                              const gchar *message,
-                             ECalClientView *client_view)
+                             GWeakRef *client_view_weak_ref)
 {
-	ECalClient *client;
-	GSource *idle_source;
-	GMainContext *main_context;
-	SignalClosure *signal_closure;
+	ECalClientView *client_view;
 
-	if (!client_view->priv->running)
-		return;
+	client_view = g_weak_ref_get (client_view_weak_ref);
 
-	signal_closure = g_slice_new0 (SignalClosure);
-	g_weak_ref_set (&signal_closure->client_view, client_view);
-	signal_closure->message = g_strdup (message);
-	signal_closure->percent = percent;
+	if (client_view != NULL) {
+		GSource *idle_source;
+		GMainContext *main_context;
+		SignalClosure *signal_closure;
 
-	client = e_cal_client_view_get_client (client_view);
-	main_context = e_client_ref_main_context (E_CLIENT (client));
+		if (!client_view->priv->running) {
+			g_object_unref (client_view);
+			return;
+		}
 
-	idle_source = g_idle_source_new ();
-	g_source_set_callback (
-		idle_source,
-		cal_client_view_emit_progress_idle_cb,
-		signal_closure,
-		(GDestroyNotify) signal_closure_free);
-	g_source_attach (idle_source, main_context);
-	g_source_unref (idle_source);
+		signal_closure = g_slice_new0 (SignalClosure);
+		g_weak_ref_set (&signal_closure->client_view, client_view);
+		signal_closure->message = g_strdup (message);
+		signal_closure->percent = percent;
 
-	g_main_context_unref (main_context);
+		main_context = cal_client_view_ref_main_context (client_view);
+
+		idle_source = g_idle_source_new ();
+		g_source_set_callback (
+			idle_source,
+			cal_client_view_emit_progress_idle_cb,
+			signal_closure,
+			(GDestroyNotify) signal_closure_free);
+		g_source_attach (idle_source, main_context);
+		g_source_unref (idle_source);
+
+		g_main_context_unref (main_context);
+
+		g_object_unref (client_view);
+	}
 }
 
 static void
 cal_client_view_complete_cb (EGdbusCalView *dbus_proxy,
                              const gchar * const *arg_error,
-                             ECalClientView *client_view)
+                             GWeakRef *client_view_weak_ref)
 {
-	ECalClient *client;
-	GSource *idle_source;
-	GMainContext *main_context;
-	SignalClosure *signal_closure;
+	ECalClientView *client_view;
 
-	if (!client_view->priv->running)
-		return;
+	client_view = g_weak_ref_get (client_view_weak_ref);
 
-	signal_closure = g_slice_new0 (SignalClosure);
-	g_weak_ref_set (&signal_closure->client_view, client_view);
-	e_gdbus_templates_decode_error (arg_error, &signal_closure->error);
+	if (client_view != NULL) {
+		GSource *idle_source;
+		GMainContext *main_context;
+		SignalClosure *signal_closure;
 
-	client = e_cal_client_view_get_client (client_view);
-	main_context = e_client_ref_main_context (E_CLIENT (client));
+		if (!client_view->priv->running) {
+			g_object_unref (client_view);
+			return;
+		}
 
-	idle_source = g_idle_source_new ();
-	g_source_set_callback (
-		idle_source,
-		cal_client_view_emit_complete_idle_cb,
-		signal_closure,
-		(GDestroyNotify) signal_closure_free);
-	g_source_attach (idle_source, main_context);
-	g_source_unref (idle_source);
+		signal_closure = g_slice_new0 (SignalClosure);
+		g_weak_ref_set (&signal_closure->client_view, client_view);
+		e_gdbus_templates_decode_error (
+			arg_error, &signal_closure->error);
 
-	g_main_context_unref (main_context);
+		main_context = cal_client_view_ref_main_context (client_view);
+
+		idle_source = g_idle_source_new ();
+		g_source_set_callback (
+			idle_source,
+			cal_client_view_emit_complete_idle_cb,
+			signal_closure,
+			(GDestroyNotify) signal_closure_free);
+		g_source_attach (idle_source, main_context);
+		g_source_unref (idle_source);
+
+		g_main_context_unref (main_context);
+
+		g_object_unref (client_view);
+	}
 }
 
 static void
@@ -426,14 +520,15 @@ cal_client_view_dispose_cb (GObject *source_object,
                             GAsyncResult *result,
                             gpointer user_data)
 {
-	GError *error = NULL;
+	GError *local_error = NULL;
 
 	e_gdbus_cal_view_call_dispose_finish (
-		G_DBUS_PROXY (source_object), result, &error);
+		G_DBUS_PROXY (source_object), result, &local_error);
 
-	if (error != NULL) {
-		g_warning ("%s: %s", G_STRFUNC, error->message);
-		g_error_free (error);
+	if (local_error != NULL) {
+		g_dbus_error_strip_remote_error (local_error);
+		g_warning ("%s: %s", G_STRFUNC, local_error->message);
+		g_error_free (local_error);
 	}
 }
 
@@ -545,6 +640,11 @@ cal_client_view_dispose (GObject *object)
 		priv->connection = NULL;
 	}
 
+	if (priv->main_context != NULL) {
+		g_main_context_unref (priv->main_context);
+		priv->main_context = NULL;
+	}
+
 	if (priv->dbus_proxy != NULL) {
 		g_signal_handler_disconnect (
 			priv->dbus_proxy,
@@ -584,6 +684,8 @@ cal_client_view_finalize (GObject *object)
 
 	g_free (priv->object_path);
 
+	g_mutex_clear (&priv->main_context_lock);
+
 	/* Chain up to parent's finalize() method. */
 	G_OBJECT_CLASS (e_cal_client_view_parent_class)->finalize (object);
 }
@@ -611,29 +713,39 @@ cal_client_view_initable_init (GInitable *initable,
 
 	priv->dbus_proxy = G_DBUS_PROXY (gdbus_calview);
 
-	handler_id = g_signal_connect (
+	handler_id = g_signal_connect_data (
 		priv->dbus_proxy, "objects-added",
-		G_CALLBACK (cal_client_view_objects_added_cb), initable);
+		G_CALLBACK (cal_client_view_objects_added_cb),
+		weak_ref_new (initable),
+		(GClosureNotify) weak_ref_free, 0);
 	priv->objects_added_handler_id = handler_id;
 
-	handler_id = g_signal_connect (
+	handler_id = g_signal_connect_data (
 		priv->dbus_proxy, "objects-modified",
-		G_CALLBACK (cal_client_view_objects_modified_cb), initable);
+		G_CALLBACK (cal_client_view_objects_modified_cb),
+		weak_ref_new (initable),
+		(GClosureNotify) weak_ref_free, 0);
 	priv->objects_modified_handler_id = handler_id;
 
-	handler_id = g_signal_connect (
+	handler_id = g_signal_connect_data (
 		priv->dbus_proxy, "objects-removed",
-		G_CALLBACK (cal_client_view_objects_removed_cb), initable);
+		G_CALLBACK (cal_client_view_objects_removed_cb),
+		weak_ref_new (initable),
+		(GClosureNotify) weak_ref_free, 0);
 	priv->objects_removed_handler_id = handler_id;
 
-	handler_id = g_signal_connect (
+	handler_id = g_signal_connect_data (
 		priv->dbus_proxy, "progress",
-		G_CALLBACK (cal_client_view_progress_cb), initable);
+		G_CALLBACK (cal_client_view_progress_cb),
+		weak_ref_new (initable),
+		(GClosureNotify) weak_ref_free, 0);
 	priv->progress_handler_id = handler_id;
 
-	handler_id = g_signal_connect (
+	handler_id = g_signal_connect_data (
 		priv->dbus_proxy, "complete",
-		G_CALLBACK (cal_client_view_complete_cb), initable);
+		G_CALLBACK (cal_client_view_complete_cb),
+		weak_ref_new (initable),
+		(GClosureNotify) weak_ref_free, 0);
 	priv->complete_handler_id = handler_id;
 
 	return TRUE;
@@ -767,6 +879,8 @@ static void
 e_cal_client_view_init (ECalClientView *view)
 {
 	view->priv = E_CAL_CLIENT_VIEW_GET_PRIVATE (view);
+
+	g_mutex_init (&view->priv->main_context_lock);
 }
 
 /**
@@ -853,20 +967,26 @@ void
 e_cal_client_view_start (ECalClientView *view,
                          GError **error)
 {
-	gboolean success;
+	GMainContext *main_context;
 	GError *local_error = NULL;
 
 	g_return_if_fail (E_IS_CAL_CLIENT_VIEW (view));
 
+	/* Emit signals from the current thread-default main context. */
+	main_context = g_main_context_ref_thread_default ();
+	cal_client_view_set_main_context (view, main_context);
+	g_main_context_unref (main_context);
+
 	view->priv->running = TRUE;
 
-	success = e_gdbus_cal_view_call_start_sync (
+	e_gdbus_cal_view_call_start_sync (
 		view->priv->dbus_proxy, NULL, &local_error);
-	if (!success)
-		view->priv->running = FALSE;
 
-	e_client_unwrap_dbus_error (
-		E_CLIENT (view->priv->client), local_error, error);
+	if (local_error != NULL) {
+		view->priv->running = FALSE;
+		g_dbus_error_strip_remote_error (local_error);
+		g_propagate_error (error, local_error);
+	}
 }
 
 /**
@@ -891,8 +1011,10 @@ e_cal_client_view_stop (ECalClientView *view,
 	e_gdbus_cal_view_call_stop_sync (
 		view->priv->dbus_proxy, NULL, &local_error);
 
-	e_client_unwrap_dbus_error (
-		E_CLIENT (view->priv->client), local_error, error);
+	if (local_error != NULL) {
+		g_dbus_error_strip_remote_error (local_error);
+		g_propagate_error (error, local_error);
+	}
 }
 
 /**
@@ -931,8 +1053,10 @@ e_cal_client_view_set_fields_of_interest (ECalClientView *view,
 		NULL, &local_error);
 	g_strfreev (strv);
 
-	e_client_unwrap_dbus_error (
-		E_CLIENT (view->priv->client), local_error, error);
+	if (local_error != NULL) {
+		g_dbus_error_strip_remote_error (local_error);
+		g_propagate_error (error, local_error);
+	}
 }
 
 /**
@@ -957,6 +1081,8 @@ e_cal_client_view_set_flags (ECalClientView *view,
 	e_gdbus_cal_view_call_set_flags_sync (
 		view->priv->dbus_proxy, flags, NULL, &local_error);
 
-	e_client_unwrap_dbus_error (
-		E_CLIENT (view->priv->client), local_error, error);
+	if (local_error != NULL) {
+		g_dbus_error_strip_remote_error (local_error);
+		g_propagate_error (error, local_error);
+	}
 }
