@@ -134,10 +134,10 @@ imapx_parse_flags (CamelIMAPXStream *stream,
 			found:
 				g_free (upper);
 			} else if (tok != ')') {
-				g_set_error (error, CAMEL_IMAPX_ERROR, 1, "expecting flag");
-				return;
+				if (!camel_imapx_stream_skip_until (stream, ")", cancellable, NULL))
+					break;
 			}
-		} while (tok != ')');
+		} while (tok != ')' && tok > 0);
 	} else {
 		g_set_error (error, CAMEL_IMAPX_ERROR, 1, "execting flag list");
 		return;
@@ -373,7 +373,8 @@ struct {
 	{ "QRESYNC", IMAPX_CAPABILITY_QRESYNC },
 	{ "LIST-EXTENDED", IMAPX_CAPABILITY_LIST_EXTENDED },
 	{ "LIST-STATUS", IMAPX_CAPABILITY_LIST_STATUS },
-	{ "QUOTA", IMAPX_CAPABILITY_QUOTA }
+	{ "QUOTA", IMAPX_CAPABILITY_QUOTA },
+	{ "MOVE", IMAPX_CAPABILITY_MOVE }
 };
 
 static GMutex capa_htable_lock;         /* capabilities lookup table lock */
@@ -1834,6 +1835,27 @@ imapx_parse_status (CamelIMAPXStream *is,
 			case IMAPX_CAPABILITY:
 				sinfo->u.cinfo = imapx_parse_capability (is, cancellable, NULL);
 				break;
+
+			/* RFC 5530 Response Codes */
+			case IMAPX_ALREADYEXISTS:
+			case IMAPX_AUTHENTICATIONFAILED:
+			case IMAPX_AUTHORIZATIONFAILED:
+			case IMAPX_CANNOT:
+			case IMAPX_CLIENTBUG:
+			case IMAPX_CONTACTADMIN:
+			case IMAPX_CORRUPTION:
+			case IMAPX_EXPIRED:
+			case IMAPX_EXPUNGEISSUED:
+			case IMAPX_INUSE:
+			case IMAPX_LIMIT:
+			case IMAPX_NONEXISTENT:
+			case IMAPX_NOPERM:
+			case IMAPX_OVERQUOTA:
+			case IMAPX_PRIVACYREQUIRED:
+			case IMAPX_SERVERBUG:
+			case IMAPX_UNAVAILABLE:
+				break;
+
 			default:
 				sinfo->condition = IMAPX_UNKNOWN;
 				d (is->tagprefix, "Got unknown response code: %s: ignored\n", token);
