@@ -36,6 +36,7 @@ struct _CamelIMAPXStore;
 /* list of strings we know about that can be *quickly* tokenised */
 typedef enum _camel_imapx_id_t {
 	IMAPX_UNKNOWN = 0,
+
 	IMAPX_ALERT,
 	IMAPX_APPENDUID,
 	IMAPX_BAD,
@@ -77,6 +78,25 @@ typedef enum _camel_imapx_id_t {
 	IMAPX_UNSEEN,
 	IMAPX_UIDNEXT,
 	IMAPX_VANISHED,
+
+	/* RFC 5530: IMAP Response Codes */
+	IMAPX_ALREADYEXISTS,
+	IMAPX_AUTHENTICATIONFAILED,
+	IMAPX_AUTHORIZATIONFAILED,
+	IMAPX_CANNOT,
+	IMAPX_CLIENTBUG,
+	IMAPX_CONTACTADMIN,
+	IMAPX_CORRUPTION,
+	IMAPX_EXPIRED,
+	IMAPX_EXPUNGEISSUED,
+	IMAPX_INUSE,
+	IMAPX_LIMIT,
+	IMAPX_NONEXISTENT,
+	IMAPX_NOPERM,
+	IMAPX_OVERQUOTA,
+	IMAPX_PRIVACYREQUIRED,
+	IMAPX_SERVERBUG,
+	IMAPX_UNAVAILABLE
 } camel_imapx_id_t;
 
 #define CAMEL_IMAPX_UNTAGGED_BAD        "BAD"
@@ -136,6 +156,14 @@ void		imapx_set_message_info_flags_for_new_message
 void		imapx_update_store_summary	(CamelFolder *folder);
 
 /* ********************************************************************** */
+
+/* Handy server capability test macros.
+ * Both return FALSE if capabilities are unknown. */
+#define CAMEL_IMAPX_HAVE_CAPABILITY(info, name) \
+	((info) != NULL && ((info)->capa & IMAPX_CAPABILITY_##name) != 0)
+#define CAMEL_IMAPX_LACK_CAPABILITY(info, name) \
+	((info) != NULL && ((info)->capa & IMAPX_CAPABILITY_##name) == 0)
+
 enum {
 	IMAPX_CAPABILITY_IMAP4			= (1 << 0),
 	IMAPX_CAPABILITY_IMAP4REV1		= (1 << 1),
@@ -149,7 +177,8 @@ enum {
 	IMAPX_CAPABILITY_QRESYNC		= (1 << 9),
 	IMAPX_CAPABILITY_LIST_STATUS		= (1 << 10),
 	IMAPX_CAPABILITY_LIST_EXTENDED		= (1 << 11),
-	IMAPX_CAPABILITY_QUOTA			= (1 << 12)
+	IMAPX_CAPABILITY_QUOTA			= (1 << 12),
+	IMAPX_CAPABILITY_MOVE			= (1 << 13)
 };
 
 struct _capability_info {
@@ -374,10 +403,13 @@ extern gint camel_imapx_debug_flags;
 #define camel_debug_flag(type) \
 	(camel_imapx_debug_flags & \
 	CAMEL_IMAPX_DEBUG_ALL & CAMEL_IMAPX_DEBUG_ ## type)
-#define camel_imapx_debug(type, tagprefix, fmt, ...) do {		\
-		if (camel_debug_flag (type))				\
+#define camel_imapx_debug(type, tagprefix, fmt, ...) \
+	G_STMT_START { \
+		if (camel_debug_flag (type)) { \
 			printf ("[imapx:%c] " fmt, tagprefix , ##__VA_ARGS__); \
-	} while (0)
+			fflush (stdout); \
+		} \
+	} G_STMT_END
 
 /* ********************************************************************** */
 
