@@ -412,6 +412,7 @@ authentication_session_execute_sync (EAuthenticationSession *session,
 	const gchar *prompt_password;
 	gchar *stored_password = NULL;
 	gboolean success;
+	gboolean remember_password = TRUE;
 	GError *local_error = NULL;
 
 	/* XXX I moved the execute() operation into a class method thinking
@@ -488,9 +489,19 @@ authentication_session_execute_sync (EAuthenticationSession *session,
 	 * has not yet been submitted to the D-Bus registry service. */
 	source = e_source_registry_server_ref_source (server, source_uid);
 	if (source != NULL) {
+		ESourceExtension *extension;
+		const gchar *extension_name;
+
 		allow_auth_prompt =
 			e_server_side_source_get_allow_auth_prompt (
 			E_SERVER_SIDE_SOURCE (source));
+
+		extension_name = E_SOURCE_EXTENSION_AUTHENTICATION;
+		extension = e_source_get_extension (source, extension_name);
+		remember_password =
+			e_source_authentication_get_remember_password (
+			E_SOURCE_AUTHENTICATION (extension));
+
 		g_object_unref (source);
 	} else {
 		allow_auth_prompt = TRUE;
@@ -531,7 +542,7 @@ authentication_session_execute_sync (EAuthenticationSession *session,
 
 	label = _("Add this password to your keyring");
 	gcr_prompt_set_choice_label (prompt, label);
-	gcr_prompt_set_choice_chosen (prompt, TRUE);
+	gcr_prompt_set_choice_chosen (prompt, remember_password);
 
 try_again:
 
@@ -553,6 +564,21 @@ try_again:
 		goto close_prompt;
 	}
 
+	source = e_source_registry_server_ref_source (server, source_uid);
+	if (source != NULL) {
+		ESourceExtension *extension;
+		const gchar *extension_name;
+
+		extension_name = E_SOURCE_EXTENSION_AUTHENTICATION;
+		extension = e_source_get_extension (source, extension_name);
+
+		e_source_authentication_set_remember_password (
+			E_SOURCE_AUTHENTICATION (extension),
+			gcr_prompt_get_choice_chosen (prompt));
+
+		g_object_unref (source);
+	}
+
 	/* Attempt authentication with the provided password. */
 
 	password_string = g_string_new (prompt_password);
