@@ -1253,6 +1253,13 @@ e_source_registry_server_class_init (ESourceRegistryServerClass *class)
 	EDBusServerClass *dbus_server_class;
 	EDataFactoryClass *data_factory_class;
 	GType backend_factory_type;
+	const gchar *modules_directory = MODULE_DIRECTORY;
+	const gchar *modules_directory_env;
+
+	modules_directory_env = g_getenv (EDS_REGISTRY_MODULES);
+	if (modules_directory_env &&
+	    g_file_test (modules_directory_env, G_FILE_TEST_IS_DIR))
+		modules_directory = g_strdup (modules_directory_env);
 
 	g_type_class_add_private (class, sizeof (ESourceRegistryServerPrivate));
 
@@ -1262,7 +1269,7 @@ e_source_registry_server_class_init (ESourceRegistryServerClass *class)
 
 	dbus_server_class = E_DBUS_SERVER_CLASS (class);
 	dbus_server_class->bus_name = SOURCES_DBUS_SERVICE_NAME;
-	dbus_server_class->module_directory = MODULE_DIRECTORY;
+	dbus_server_class->module_directory = modules_directory;
 	dbus_server_class->bus_acquired = source_registry_server_bus_acquired;
 	dbus_server_class->quit_server = source_registry_server_quit_server;
 
@@ -1778,20 +1785,38 @@ e_source_registry_server_load_directory (ESourceRegistryServer *server,
 	 * sources directory should be removable. */
 	if (removable) {
 		GFileMonitor *monitor;
+		GError *local_error = NULL;
 
+		/* Directory monitoring is a nice-to-have feature.
+		 * If this fails, leave a breadcrumb on the console
+		 * to indicate something went wrong, but don't return
+		 * an error status. */
 		monitor = g_file_monitor_directory (
-			file, G_FILE_MONITOR_NONE, NULL, error);
-		if (monitor == NULL)
-			return FALSE;
+			file, G_FILE_MONITOR_NONE, NULL, &local_error);
 
-		g_signal_connect (
-			monitor, "changed",
-			G_CALLBACK (source_registry_server_monitor_changed_cb),
-			server);
+		/* Sanity check. */
+		g_warn_if_fail (
+			((monitor != NULL) && (local_error == NULL)) ||
+			((monitor == NULL) && (local_error != NULL)));
 
-		g_hash_table_insert (
-			server->priv->monitors,
-			g_object_ref (file), monitor);
+		if (monitor != NULL) {
+			g_signal_connect (
+				monitor, "changed", G_CALLBACK (
+				source_registry_server_monitor_changed_cb),
+				server);
+
+			g_hash_table_insert (
+				server->priv->monitors,
+				g_object_ref (file),
+				g_object_ref (monitor));
+
+			g_object_unref (monitor);
+		}
+
+		if (local_error != NULL) {
+			g_warning ("%s: %s", G_STRFUNC, local_error->message);
+			g_error_free (local_error);
+		}
 	}
 
 	g_object_unref (file);
@@ -2331,8 +2356,9 @@ e_source_registry_server_new_auth_session (ESourceRegistryServer *server,
 	}
 
 	g_return_val_if_fail (
-		g_type_is_a (auth_session_type,
-		E_TYPE_AUTHENTICATION_SESSION), NULL);
+		g_type_is_a (
+			auth_session_type,
+			E_TYPE_AUTHENTICATION_SESSION), NULL);
 
 	return g_object_new (
 		auth_session_type,
