@@ -54,6 +54,7 @@ struct _EClientPrivate {
 	gboolean online;
 	gboolean readonly;
 	GSList *capabilities;
+	GMainContext *main_context;
 };
 
 struct _AsyncContext {
@@ -65,11 +66,12 @@ struct _AsyncContext {
 
 enum {
 	PROP_0,
-	PROP_SOURCE,
 	PROP_CAPABILITIES,
-	PROP_READONLY,
+	PROP_MAIN_CONTEXT,
 	PROP_ONLINE,
-	PROP_OPENED
+	PROP_OPENED,
+	PROP_READONLY,
+	PROP_SOURCE
 };
 
 enum {
@@ -140,8 +142,9 @@ e_client_error_to_string (EClientError code)
 	case E_CLIENT_ERROR_REPOSITORY_OFFLINE:
 		return _("Repository offline");
 	case E_CLIENT_ERROR_OFFLINE_UNAVAILABLE:
-		/* Translators: This means that the EClient does not support offline mode, or
-		 * it's not set to by a user, thus it is unavailable while user is not connected. */
+		/* Translators: This means that the EClient does not
+		 * support offline mode, or it's not set to by a user,
+		 * thus it is unavailable while user is not connected. */
 		return _("Offline unavailable");
 	case E_CLIENT_ERROR_PERMISSION_DENIED:
 		return _("Permission denied");
@@ -189,54 +192,27 @@ e_client_error_to_string (EClientError code)
  * Returned pointer should be freed with g_error_free().
  *
  * Since: 3.2
+ *
+ * Deprecated: 3.8: Just use the #GError API directly.
  **/
 GError *
 e_client_error_create (EClientError code,
                        const gchar *custom_msg)
 {
-	return g_error_new_literal (E_CLIENT_ERROR, code, custom_msg ? custom_msg : e_client_error_to_string (code));
-}
+	if (custom_msg == NULL)
+		custom_msg = e_client_error_to_string (code);
 
-static void client_set_source (EClient *client, ESource *source);
-
-static void
-e_client_init (EClient *client)
-{
-	client->priv = E_CLIENT_GET_PRIVATE (client);
-
-	client->priv->readonly = TRUE;
-
-	g_rec_mutex_init (&client->priv->prop_mutex);
+	return g_error_new_literal (E_CLIENT_ERROR, code, custom_msg);
 }
 
 static void
-client_finalize (GObject *object)
+client_set_source (EClient *client,
+                   ESource *source)
 {
-	EClient *client;
-	EClientPrivate *priv;
+	g_return_if_fail (E_IS_SOURCE (source));
+	g_return_if_fail (client->priv->source == NULL);
 
-	client = E_CLIENT (object);
-
-	priv = client->priv;
-
-	g_rec_mutex_lock (&priv->prop_mutex);
-
-	if (priv->source) {
-		g_object_unref (priv->source);
-		priv->source = NULL;
-	}
-
-	if (priv->capabilities) {
-		g_slist_foreach (priv->capabilities, (GFunc) g_free, NULL);
-		g_slist_free (priv->capabilities);
-		priv->capabilities = NULL;
-	}
-
-	g_rec_mutex_unlock (&priv->prop_mutex);
-	g_rec_mutex_clear (&priv->prop_mutex);
-
-	/* Chain up to parent's finalize() method. */
-	G_OBJECT_CLASS (e_client_parent_class)->finalize (object);
+	client->priv->source = g_object_ref (source);
 }
 
 static void
@@ -246,17 +222,17 @@ client_set_property (GObject *object,
                      GParamSpec *pspec)
 {
 	switch (property_id) {
-		case PROP_SOURCE:
-			client_set_source (
-				E_CLIENT (object),
-				g_value_get_object (value));
-			return;
-
 		case PROP_ONLINE:
 			e_client_set_online (
 				E_CLIENT (object),
 				g_value_get_boolean (value));
 			return;
+
+		case PROP_SOURCE:
+			client_set_source (
+				E_CLIENT (object),
+				g_value_get_object (value));
+			return;
 	}
 
 	G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
@@ -269,13 +245,6 @@ client_get_property (GObject *object,
                      GParamSpec *pspec)
 {
 	switch (property_id) {
-		case PROP_SOURCE:
-			g_value_set_object (
-				value,
-				e_client_get_source (
-				E_CLIENT (object)));
-			return;
-
 		case PROP_CAPABILITIES:
 			g_value_set_pointer (
 				value,
@@ -283,10 +252,10 @@ client_get_property (GObject *object,
 				E_CLIENT (object)));
 			return;
 
-		case PROP_READONLY:
-			g_value_set_boolean (
+		case PROP_MAIN_CONTEXT:
+			g_value_take_boxed (
 				value,
-				e_client_is_readonly (
+				e_client_ref_main_context (
 				E_CLIENT (object)));
 			return;
 
@@ -303,11 +272,69 @@ client_get_property (GObject *object,
 				e_client_is_opened (
 				E_CLIENT (object)));
 			return;
+
+		case PROP_READONLY:
+			g_value_set_boolean (
+				value,
+				e_client_is_readonly (
+				E_CLIENT (object)));
+			return;
+
+		case PROP_SOURCE:
+			g_value_set_object (
+				value,
+				e_client_get_source (
+				E_CLIENT (object)));
+			return;
 	}
 
 	G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
 }
 
+static void
+client_dispose (GObject *object)
+{
+	EClientPrivate *priv;
+
+	priv = E_CLIENT_GET_PRIVATE (object);
+
+	if (priv->main_context != NULL) {
+		g_main_context_unref (priv->main_context);
+		priv->main_context = NULL;
+	}
+
+	g_clear_object (&priv->source);
+
+	/* Chain up to parent's dispose() method. */
+	G_OBJECT_CLASS (e_client_parent_class)->dispose (object);
+}
+
+static void
+client_finalize (GObject *object)
+{
+	EClientPrivate *priv;
+
+	priv = E_CLIENT_GET_PRIVATE (object);
+
+	g_slist_free_full (priv->capabilities, (GDestroyNotify) g_free);
+
+	g_rec_mutex_clear (&priv->prop_mutex);
+
+	/* Chain up to parent's finalize() method. */
+	G_OBJECT_CLASS (e_client_parent_class)->finalize (object);
+}
+
+static void
+client_unwrap_dbus_error (EClient *client,
+                          GError *dbus_error,
+                          GError **out_error)
+{
+	/* This method is deprecated.  Make it a no-op. */
+
+	if (out_error != NULL)
+		*out_error = dbus_error;
+}
+
 /* Helper for client_retrieve_capabilities() */
 static void
 client_retrieve_capabilities_thread (GSimpleAsyncResult *simple,
@@ -737,8 +764,10 @@ e_client_class_init (EClientClass *class)
 	object_class = G_OBJECT_CLASS (class);
 	object_class->set_property = client_set_property;
 	object_class->get_property = client_get_property;
+	object_class->dispose = client_dispose;
 	object_class->finalize = client_finalize;
 
+	class->unwrap_dbus_error = client_unwrap_dbus_error;
 	class->retrieve_capabilities = client_retrieve_capabilities;
 	class->retrieve_capabilities_finish = client_retrieve_capabilities_finish;
 	class->retrieve_capabilities_sync = client_retrieve_capabilities_sync;
@@ -754,18 +783,6 @@ e_client_class_init (EClientClass *class)
 	class->refresh = client_refresh;
 	class->refresh_finish = client_refresh_finish;
 
-	g_object_class_install_property (
-		object_class,
-		PROP_SOURCE,
-		g_param_spec_object (
-			"source",
-			NULL,
-			NULL,
-			E_TYPE_SOURCE,
-			G_PARAM_READWRITE |
-			G_PARAM_CONSTRUCT_ONLY |
-			G_PARAM_STATIC_STRINGS));
-
 	g_object_class_install_property (
 		object_class,
 		PROP_CAPABILITIES,
@@ -778,12 +795,13 @@ e_client_class_init (EClientClass *class)
 
 	g_object_class_install_property (
 		object_class,
-		PROP_READONLY,
-		g_param_spec_boolean (
-			"readonly",
-			NULL,
-			NULL,
-			FALSE,
+		PROP_MAIN_CONTEXT,
+		g_param_spec_boxed (
+			"main-context",
+			"Main Context",
+			"The main loop context on "
+			"which to attach event sources",
+			G_TYPE_MAIN_CONTEXT,
 			G_PARAM_READABLE |
 			G_PARAM_STATIC_STRINGS));
 
@@ -809,6 +827,29 @@ e_client_class_init (EClientClass *class)
 			G_PARAM_READABLE |
 			G_PARAM_STATIC_STRINGS));
 
+	g_object_class_install_property (
+		object_class,
+		PROP_READONLY,
+		g_param_spec_boolean (
+			"readonly",
+			NULL,
+			NULL,
+			FALSE,
+			G_PARAM_READABLE |
+			G_PARAM_STATIC_STRINGS));
+
+	g_object_class_install_property (
+		object_class,
+		PROP_SOURCE,
+		g_param_spec_object (
+			"source",
+			NULL,
+			NULL,
+			E_TYPE_SOURCE,
+			G_PARAM_READWRITE |
+			G_PARAM_CONSTRUCT_ONLY |
+			G_PARAM_STATIC_STRINGS));
+
 	/**
 	 * EClient::opened:
 	 *
@@ -855,18 +896,14 @@ e_client_class_init (EClientClass *class)
 }
 
 static void
-client_set_source (EClient *client,
-                   ESource *source)
+e_client_init (EClient *client)
 {
-	g_return_if_fail (E_IS_CLIENT (client));
-	g_return_if_fail (E_IS_SOURCE (source));
+	client->priv = E_CLIENT_GET_PRIVATE (client);
 
-	g_object_ref (source);
+	client->priv->readonly = TRUE;
+	client->priv->main_context = g_main_context_ref_thread_default ();
 
-	if (client->priv->source)
-		g_object_unref (client->priv->source);
-
-	client->priv->source = source;
+	g_rec_mutex_init (&client->priv->prop_mutex);
 }
 
 /**
@@ -928,6 +965,28 @@ e_client_get_capabilities (EClient *client)
 	return client->priv->capabilities;
 }
 
+/**
+ * e_client_ref_main_context:
+ * @client: an #EClient
+ *
+ * Returns the #GMainContext on which event sources for @client are to
+ * be attached.
+ *
+ * The returned #GMainContext is referenced for thread-safety and must be
+ * unreferenced with g_main_context_unref() when finished with it.
+ *
+ * Returns: (transfer full): a #GMainContext
+ *
+ * Since: 3.8
+ **/
+GMainContext *
+e_client_ref_main_context (EClient *client)
+{
+	g_return_val_if_fail (E_IS_CLIENT (client), NULL);
+
+	return g_main_context_ref (client->priv->main_context);
+}
+
 /**
  * e_client_check_capability:
  * @client: an #EClient
@@ -1192,13 +1251,11 @@ e_client_retrieve_capabilities_finish (EClient *client,
 	g_return_val_if_fail (class->retrieve_capabilities_finish != NULL, FALSE);
 
 	*capabilities = NULL;
-	res = class->retrieve_capabilities_finish (client, result, capabilities, error);
+	res = class->retrieve_capabilities_finish (
+		client, result, capabilities, error);
 
 	e_client_set_capabilities (client, res ? *capabilities : NULL);
 
-	if (error && *error)
-		e_client_unwrap_dbus_error (client, *error, error);
-
 	return res;
 }
 
@@ -1238,13 +1295,11 @@ e_client_retrieve_capabilities_sync (EClient *client,
 	g_return_val_if_fail (class->retrieve_capabilities_sync != NULL, FALSE);
 
 	*capabilities = NULL;
-	res = class->retrieve_capabilities_sync (client, capabilities, cancellable, error);
+	res = class->retrieve_capabilities_sync (
+		client, capabilities, cancellable, error);
 
 	e_client_set_capabilities (client, res ? *capabilities : NULL);
 
-	if (error && *error)
-		e_client_unwrap_dbus_error (client, *error, error);
-
 	return res;
 }
 
@@ -1279,7 +1334,8 @@ e_client_get_backend_property (EClient *client,
 	g_return_if_fail (class != NULL);
 	g_return_if_fail (class->get_backend_property != NULL);
 
-	class->get_backend_property (client, prop_name, cancellable, callback, user_data);
+	class->get_backend_property (
+		client, prop_name, cancellable, callback, user_data);
 }
 
 /**
@@ -1302,7 +1358,6 @@ e_client_get_backend_property_finish (EClient *client,
                                       GError **error)
 {
 	EClientClass *class;
-	gboolean res;
 
 	g_return_val_if_fail (E_IS_CLIENT (client), FALSE);
 	g_return_val_if_fail (prop_value != NULL, FALSE);
@@ -1311,12 +1366,8 @@ e_client_get_backend_property_finish (EClient *client,
 	g_return_val_if_fail (class != NULL, FALSE);
 	g_return_val_if_fail (class->get_backend_property_finish != NULL, FALSE);
 
-	res = class->get_backend_property_finish (client, result, prop_value, error);
-
-	if (error && *error)
-		e_client_unwrap_dbus_error (client, *error, error);
-
-	return res;
+	return class->get_backend_property_finish (
+		client, result, prop_value, error);
 }
 
 /**
@@ -1341,7 +1392,6 @@ e_client_get_backend_property_sync (EClient *client,
                                     GError **error)
 {
 	EClientClass *class;
-	gboolean res;
 
 	g_return_val_if_fail (E_IS_CLIENT (client), FALSE);
 	g_return_val_if_fail (prop_name != NULL, FALSE);
@@ -1351,12 +1401,8 @@ e_client_get_backend_property_sync (EClient *client,
 	g_return_val_if_fail (class != NULL, FALSE);
 	g_return_val_if_fail (class->get_backend_property_sync != NULL, FALSE);
 
-	res = class->get_backend_property_sync (client, prop_name, prop_value, cancellable, error);
-
-	if (error && *error)
-		e_client_unwrap_dbus_error (client, *error, error);
-
-	return res;
+	return class->get_backend_property_sync (
+		client, prop_name, prop_value, cancellable, error);
 }
 
 /**
@@ -1396,7 +1442,9 @@ e_client_set_backend_property (EClient *client,
 	g_return_if_fail (class != NULL);
 	g_return_if_fail (class->set_backend_property != NULL);
 
-	class->set_backend_property (client, prop_name, prop_value, cancellable, callback, user_data);
+	class->set_backend_property (
+		client, prop_name, prop_value,
+		cancellable, callback, user_data);
 }
 
 /**
@@ -1420,7 +1468,6 @@ e_client_set_backend_property_finish (EClient *client,
                                       GError **error)
 {
 	EClientClass *class;
-	gboolean res;
 
 	g_return_val_if_fail (E_IS_CLIENT (client), FALSE);
 
@@ -1428,12 +1475,7 @@ e_client_set_backend_property_finish (EClient *client,
 	g_return_val_if_fail (class != NULL, FALSE);
 	g_return_val_if_fail (class->set_backend_property_finish != NULL, FALSE);
 
-	res = class->set_backend_property_finish (client, result, error);
-
-	if (error && *error)
-		e_client_unwrap_dbus_error (client, *error, error);
-
-	return res;
+	return class->set_backend_property_finish (client, result, error);
 }
 
 /**
@@ -1462,7 +1504,6 @@ e_client_set_backend_property_sync (EClient *client,
                                     GError **error)
 {
 	EClientClass *class;
-	gboolean res;
 
 	g_return_val_if_fail (E_IS_CLIENT (client), FALSE);
 	g_return_val_if_fail (prop_name != NULL, FALSE);
@@ -1472,18 +1513,15 @@ e_client_set_backend_property_sync (EClient *client,
 	g_return_val_if_fail (class != NULL, FALSE);
 	g_return_val_if_fail (class->set_backend_property_sync != NULL, FALSE);
 
-	res = class->set_backend_property_sync (client, prop_name, prop_value, cancellable, error);
-
-	if (error && *error)
-		e_client_unwrap_dbus_error (client, *error, error);
-
-	return res;
+	return class->set_backend_property_sync (
+		client, prop_name, prop_value, cancellable, error);
 }
 
 /**
  * e_client_open:
  * @client: an #EClient
- * @only_if_exists: if %TRUE, fail if this book doesn't already exist, otherwise create it first
+ * @only_if_exists: if %TRUE, fail if this book doesn't already exist,
+ *                  otherwise create it first
  * @cancellable: a #GCancellable; can be %NULL
  * @callback: callback to call when a result is ready
  * @user_data: user data for the @callback
@@ -1540,7 +1578,6 @@ e_client_open_finish (EClient *client,
                       GError **error)
 {
 	EClientClass *class;
-	gboolean res;
 
 	g_return_val_if_fail (E_IS_CLIENT (client), FALSE);
 
@@ -1548,18 +1585,14 @@ e_client_open_finish (EClient *client,
 	g_return_val_if_fail (class != NULL, FALSE);
 	g_return_val_if_fail (class->open_finish != NULL, FALSE);
 
-	res = class->open_finish (client, result, error);
-
-	if (error && *error)
-		e_client_unwrap_dbus_error (client, *error, error);
-
-	return res;
+	return class->open_finish (client, result, error);
 }
 
 /**
  * e_client_open_sync:
  * @client: an #EClient
- * @only_if_exists: if %TRUE, fail if this book doesn't already exist, otherwise create it first
+ * @only_if_exists: if %TRUE, fail if this book doesn't already exist,
+ *                  otherwise create it first
  * @cancellable: a #GCancellable; can be %NULL
  * @error: (out): a #GError to set an error, if any
  *
@@ -1579,18 +1612,12 @@ e_client_open_sync (EClient *client,
                     GError **error)
 {
 	EClientClass *class;
-	gboolean res;
 
 	class = E_CLIENT_GET_CLASS (client);
 	g_return_val_if_fail (class != NULL, FALSE);
 	g_return_val_if_fail (class->open_sync != NULL, FALSE);
 
-	res = class->open_sync (client, only_if_exists, cancellable, error);
-
-	if (error && *error)
-		e_client_unwrap_dbus_error (client, *error, error);
-
-	return res;
+	return class->open_sync (client, only_if_exists, cancellable, error);
 }
 
 /**
@@ -1646,7 +1673,6 @@ e_client_remove_finish (EClient *client,
                         GError **error)
 {
 	EClientClass *class;
-	gboolean res;
 
 	g_return_val_if_fail (E_IS_CLIENT (client), FALSE);
 
@@ -1654,12 +1680,7 @@ e_client_remove_finish (EClient *client,
 	g_return_val_if_fail (class != NULL, FALSE);
 	g_return_val_if_fail (class->remove_finish != NULL, FALSE);
 
-	res = class->remove_finish (client, result, error);
-
-	if (error && *error)
-		e_client_unwrap_dbus_error (client, *error, error);
-
-	return res;
+	return class->remove_finish (client, result, error);
 }
 
 /**
@@ -1683,18 +1704,12 @@ e_client_remove_sync (EClient *client,
                       GError **error)
 {
 	EClientClass *class;
-	gboolean res;
 
 	class = E_CLIENT_GET_CLASS (client);
 	g_return_val_if_fail (class != NULL, FALSE);
 	g_return_val_if_fail (class->remove_sync != NULL, FALSE);
 
-	res = class->remove_sync (client, cancellable, error);
-
-	if (error && *error)
-		e_client_unwrap_dbus_error (client, *error, error);
-
-	return res;
+	return class->remove_sync (client, cancellable, error);
 }
 
 /**
@@ -1748,7 +1763,6 @@ e_client_refresh_finish (EClient *client,
                          GError **error)
 {
 	EClientClass *class;
-	gboolean res;
 
 	g_return_val_if_fail (E_IS_CLIENT (client), FALSE);
 
@@ -1756,12 +1770,7 @@ e_client_refresh_finish (EClient *client,
 	g_return_val_if_fail (class != NULL, FALSE);
 	g_return_val_if_fail (class->refresh_finish != NULL, FALSE);
 
-	res = class->refresh_finish (client, result, error);
-
-	if (error && *error)
-		e_client_unwrap_dbus_error (client, *error, error);
-
-	return res;
+	return class->refresh_finish (client, result, error);
 }
 
 /**
@@ -1785,18 +1794,12 @@ e_client_refresh_sync (EClient *client,
                        GError **error)
 {
 	EClientClass *class;
-	gboolean res;
 
 	class = E_CLIENT_GET_CLASS (client);
 	g_return_val_if_fail (class != NULL, FALSE);
 	g_return_val_if_fail (class->refresh_sync != NULL, FALSE);
 
-	res = class->refresh_sync (client, cancellable, error);
-
-	if (error && *error)
-		e_client_unwrap_dbus_error (client, *error, error);
-
-	return res;
+	return class->refresh_sync (client, cancellable, error);
 }
 
 /**
@@ -1964,6 +1967,8 @@ e_client_util_parse_comma_strings (const gchar *strings)
  * @dbus_erorr and @out_error can point to the same variable.
  *
  * Since: 3.2
+ *
+ * Deprecated: 3.8: Use g_dbus_error_strip_remote_error() instead.
  **/
 void
 e_client_unwrap_dbus_error (EClient *client,
@@ -2011,6 +2016,8 @@ e_client_unwrap_dbus_error (EClient *client,
  * Note: The @dbus_error is automatically freed if returned %TRUE.
  *
  * Since: 3.2
+ *
+ * Deprecated: 3.8: This function is no longer used.
  **/
 gboolean
 e_client_util_unwrap_dbus_error (GError *dbus_error,
@@ -2048,7 +2055,10 @@ e_client_util_unwrap_dbus_error (GError *dbus_error,
 					g_free (name);
 
 					g_dbus_error_strip_remote_error (dbus_error);
-					*client_error = g_error_new_literal (known_errors_domain, known_errors[ii].err_code, dbus_error->message);
+					*client_error = g_error_new_literal (
+						known_errors_domain,
+						known_errors[ii].err_code,
+						dbus_error->message);
 					g_error_free (dbus_error);
 					return TRUE;
 				}
@@ -2063,7 +2073,10 @@ e_client_util_unwrap_dbus_error (GError *dbus_error,
 
 	if (g_error_matches (dbus_error, G_IO_ERROR, G_IO_ERROR_DBUS_ERROR)) {
 		g_dbus_error_strip_remote_error (dbus_error);
-		*client_error = g_error_new_literal (E_CLIENT_ERROR, E_CLIENT_ERROR_OTHER_ERROR, dbus_error->message);
+		*client_error = g_error_new_literal (
+			E_CLIENT_ERROR,
+			E_CLIENT_ERROR_OTHER_ERROR,
+			dbus_error->message);
 		g_error_free (dbus_error);
 	} else {
 		g_dbus_error_strip_remote_error (dbus_error);
