@@ -301,20 +301,37 @@ main (gint argc,
 	g_type_init ();
 #endif
 	g_test_init (&argc, &argv, NULL);
+	g_test_bug_base ("http://bugzilla.gnome.org/");
 
 	g_test_add (
-		"/EBookClient/UidOnlyView/Sync/AllData", ETestServerFixture, &book_closure_all_data,
-		e_test_server_utils_setup, test_get_view_sync, e_test_server_utils_teardown);
+		"/EBookClient/UidOnlyView/Sync/AllData",
+		ETestServerFixture,
+		&book_closure_all_data,
+		e_test_server_utils_setup,
+		test_get_view_sync,
+		e_test_server_utils_teardown);
 	g_test_add (
-		"/EBookClient/UidOnlyView/Sync/UidsOnly", ETestServerFixture, &book_closure_uids_only,
-		e_test_server_utils_setup, test_get_view_sync, e_test_server_utils_teardown);
+		"/EBookClient/UidOnlyView/Sync/UidsOnly",
+		ETestServerFixture,
+		&book_closure_uids_only,
+		e_test_server_utils_setup,
+		test_get_view_sync,
+		e_test_server_utils_teardown);
 
 	g_test_add (
-		"/EBookClient/UidOnlyView/Async/AllData", ETestServerFixture, &book_closure_all_data,
-		e_test_server_utils_setup, test_get_view_async, e_test_server_utils_teardown);
+		"/EBookClient/UidOnlyView/Async/AllData",
+		ETestServerFixture,
+		&book_closure_all_data,
+		e_test_server_utils_setup,
+		test_get_view_async,
+		e_test_server_utils_teardown);
 	g_test_add (
-		"/EBookClient/UidOnlyView/Async/UidsOnly", ETestServerFixture, &book_closure_uids_only,
-		e_test_server_utils_setup, test_get_view_async, e_test_server_utils_teardown);
+		"/EBookClient/UidOnlyView/Async/UidsOnly",
+		ETestServerFixture,
+		&book_closure_uids_only,
+		e_test_server_utils_setup,
+		test_get_view_async,
+		e_test_server_utils_teardown);
 
 	return e_test_server_utils_run ();
 }
